"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = addPackage;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _changeCase = require("change-case");

var _prompts = _interopRequireDefault(require("prompts"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _execSync = _interopRequireDefault(require("./utils/execSync"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const packagesRoot = 'packages'; // recursively get all files in a folder

function getAllFiles(dirPath, arrayOfFiles = []) {
  const files = fs.readdirSync(dirPath);
  files.forEach(function (file) {
    const pathToCheck = path.join(dirPath, file);

    if (fs.statSync(pathToCheck).isDirectory()) {
      arrayOfFiles = getAllFiles(pathToCheck, arrayOfFiles);
    } else {
      arrayOfFiles.push(pathToCheck);
    }
  });
  return arrayOfFiles;
}

async function addPackage(destination, typeArg, nameArg, preferOffline = true) {
  const {
    type,
    name
  } = (typeArg && nameArg ? {
    type: typeArg,
    name: nameArg
  } : null) || (await (0, _prompts.default)([{
    name: 'name',
    type: 'text',
    message: `What would you like to name this package?`,
    initial: (0, _changeCase.paramCase)(destination)
  }, {
    name: 'type',
    type: 'select',
    message: `What kind of package is this?`,
    choices: [{
      title: 'A plain package',
      value: 'package'
    }, {
      title: 'A view within an application',
      value: 'view'
    }, {
      title: 'A standalone application',
      value: 'app'
    }],
    initial: 0
  }]));

  if (!['app', 'view', 'package'].includes(type)) {
    throw new Error(`Type ${type} does not exist, please use app, view or package`);
  }

  const modularRoot = (0, _getModularRoot.default)();
  const newComponentName = (0, _changeCase.pascalCase)(name);
  const newPackagePath = path.join(modularRoot, packagesRoot, destination);
  const packageTypePath = path.join(__dirname, '../types', type); // create a new package source folder

  if (fs.existsSync(newPackagePath)) {
    console.error(`A package already exists at ${destination}!`);
    process.exit(1);
  }

  fs.mkdirpSync(newPackagePath);
  fs.copySync(packageTypePath, newPackagePath);
  const packageFilePaths = getAllFiles(newPackagePath);

  for (const packageFilePath of packageFilePaths) {
    fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, name).replace(/ComponentName__/g, newComponentName));

    if (path.basename(packageFilePath) === 'packagejson') {
      // we've named package.json as packagejson in these templates
      fs.moveSync(packageFilePath, packageFilePath.replace('packagejson', 'package.json'));
    }
  }

  if (type === 'app') {
    // add a tsconfig, because CRA expects it
    await fs.writeJSON(path.join(newPackagePath, 'tsconfig.json'), {
      extends: path.relative(newPackagePath, modularRoot) + '/tsconfig.json'
    }, {
      spaces: 2
    });
  }

  const yarnArgs = ['--silent'];

  if (preferOffline) {
    yarnArgs.push('--prefer-offline');
  }

  (0, _execSync.default)('yarnpkg', yarnArgs, {
    cwd: modularRoot
  });
}