"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = build;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _resolveAsBin = _interopRequireDefault(require("resolve-as-bin"));

var _config = require("./config");

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _isModularType = _interopRequireDefault(require("./utils/isModularType"));

var _execSync = _interopRequireDefault(require("./utils/execSync"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function build(packagePath, preserveModules) {
  const modularRoot = (0, _getModularRoot.default)();

  if ((0, _isModularType.default)(path.join(modularRoot, _config.packagesRoot, packagePath), 'app')) {
    const cracoBin = (0, _resolveAsBin.default)('craco'); // create-react-app doesn't support plain module outputs yet,
    // so --preserve-modules has no effect here

    await fs.remove(path.join(_config.outputDirectory, packagePath)); // TODO: this shouldn't be sync

    (0, _execSync.default)(cracoBin, ['build', '--config', _config.cracoConfig], {
      cwd: path.join(modularRoot, _config.packagesRoot, packagePath),
      log: false,
      // @ts-ignore
      env: {
        USE_MODULAR_BABEL: process.env.USE_MODULAR_BABEL,
        MODULAR_ROOT: modularRoot
      }
    });
    await fs.move(path.join(_config.packagesRoot, packagePath, 'build'), path.join(_config.outputDirectory, packagePath));
  } else {
    const {
      buildPackage
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./buildPackage'))); // ^ we do a dynamic import here to defer the module's initial side effects
    // till when it's actually needed (i.e. now)

    await buildPackage(packagePath, preserveModules);
  }
}