"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildPackage = buildPackage;

var _util = require("util");

var rollup = _interopRequireWildcard(require("rollup"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var path = _interopRequireWildcard(require("path"));

var _tar = require("tar");

var _execa = _interopRequireDefault(require("execa"));

var _changeCase = require("change-case");

var _rollupPluginPostcss = _interopRequireDefault(require("rollup-plugin-postcss"));

var _pluginJson = _interopRequireDefault(require("@rollup/plugin-json"));

var _pluginNodeResolve = _interopRequireDefault(require("@rollup/plugin-node-resolve"));

var _pluginBabel = _interopRequireDefault(require("@rollup/plugin-babel"));

var _pluginCommonjs = _interopRequireDefault(require("@rollup/plugin-commonjs"));

var _rollupPluginPreserveShebangs = require("rollup-plugin-preserve-shebangs");

var ts = _interopRequireWildcard(require("typescript"));

var fse = _interopRequireWildcard(require("fs-extra"));

var _builtinModules = _interopRequireDefault(require("builtin-modules"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _getAllWorkspaces = require("./utils/getAllWorkspaces");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// While working on this file, be aware that builds
// could be happening simultaneously across packages, so
// try be 'thread-safe'. No state outside of functions
// shorthand for building every workspace, if you're ever debugging this flow
// rm -rf dist && yarn modular build `ls -m1 packages | sed -e 'H;${x;s/\n/,/g;s/^,//;p;};d'`
const consoles = {};

function getConsole(directoryName) {
  if (!consoles[directoryName]) {
    consoles[directoryName] = {
      log: (...args) => {
        return console.log('$ ' + directoryName + ':', ...args);
      },
      error: (...args) => {
        return console.error('$ ' + directoryName + ':', ...args);
      }
    };
  }

  return consoles[directoryName];
} // from https://github.com/Microsoft/TypeScript/issues/6387
// a helper to output a readable message from a ts diagnostics object


function reportTSDiagnostics(packagePath, diagnostics) {
  diagnostics.forEach(diagnostic => {
    let message = 'Error';

    if (diagnostic.file) {
      const where = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
      message += ` ${diagnostic.file.fileName} ${where.line}, ${where.character + 1}`;
    }

    message += ': ' + ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
    console.log(message);
  });
}

const extensions = ['.ts', '.tsx', '.js', '.jsx'];
const outputDirectory = 'dist';
const typescriptConfigFilename = 'tsconfig.json';
const packagesRoot = 'packages'; // dependencies defined at the root

const rootPackageJsonDependencies = fse.readJSONSync('package.json').dependencies || {}; // a map of all package.json contents, indexed by package name

const packageJsons = {}; // a map of all package.json contents, indexed by directory name

const packageJsonsByPackagePath = {}; // an array of all the package names

const packageNames = []; // let's populate the above three

for (const [name, {
  location
}] of Object.entries((0, _getAllWorkspaces.getAllWorkspaces)())) {
  const pathToPackageJson = path.join(location, 'package.json');
  const packageJson = fse.readJsonSync(pathToPackageJson);
  packageJsons[name] = packageJson;
  packageJsonsByPackagePath[location.replace(new RegExp('^packages\\/'), '')] = packageJson;
  packageNames.push(name);
} // TODO: the above should probably be lazily evaluated.
// TODO: do a quick check to make sure workspaces aren't
// explicitly included in dependencies
// maybe that belongs in `modular check`


const publicPackageJsons = {};

function distinct(arr) {
  return [...new Set(arr)];
}

const typescriptConfig = {}; // validate tsconfig

{
  // Extract configuration from config file and parse JSON,
  // after removing comments. Just a fancier JSON.parse
  const result = ts.parseConfigFileTextToJson(typescriptConfigFilename, fse.readFileSync(typescriptConfigFilename, 'utf8').toString());
  const configObject = result.config;

  if (!configObject) {
    reportTSDiagnostics(':root', [result.error]);
    throw new Error('Failed to load Typescript configuration');
  }

  Object.assign(typescriptConfig, configObject, {
    // TODO: should probably include the original exclude in this
    exclude: distinct([// all TS test files, regardless whether co-located or in test/ etc
    '**/*.stories.ts', '**/*.stories.tsx', '**/*.spec.ts', '**/*.test.ts', '**/*.e2e.ts', '**/*.spec.tsx', '**/*.test.tsx', '__tests__', '**/dist-cjs', '**/dist-es', '**/dist-types', // TS defaults below
    'node_modules', 'bower_components', 'jspm_packages', 'tmp', // Casting so that configObject.exclude is set to the correct typing
    // Since configObject is a index type all values are "any" implicitly.
    ...(configObject.exclude || [])])
  });
  typescriptConfig.compilerOptions = typescriptConfig.compilerOptions || {};
  Object.assign(typescriptConfig.compilerOptions, {
    declarationDir: outputDirectory,
    noEmit: false,
    declaration: true,
    emitDeclarationOnly: true,
    incremental: false
  });
}

async function makeBundle(packagePath, preserveModules) {
  const console = getConsole(packagePath);
  const packageJson = packageJsonsByPackagePath[packagePath];

  if (!packageJson) {
    throw new Error(`no package.json in ${packagesRoot}/${packagePath}, bailing...`);
  }

  if (packageJson.private === true) {
    throw new Error(`${packagesRoot}/${packagePath} is marked private, bailing...`);
  }

  let compilingBin = false;
  let main;

  if (packageJson.main) {
    main = packageJson.main;
  } else {
    if (packageJson.bin) {
      const bins = Object.values(packageJson.bin);

      if (bins.length === 1) {
        compilingBin = true;
        main = bins[0];
      } else {
        throw new Error(`package.json at ${packagesRoot}/${packagePath} contains multiple "bin" values, bailing...`);
      }
    } else {
      throw new Error(`package.json at ${packagesRoot}/${packagePath} does not have a "main" or "bin" field, bailing...`);
    }
  }

  if (!fse.existsSync(path.join(packagesRoot, packagePath, main))) {
    throw new Error(`package.json at ${packagesRoot}/${packagePath} does not have a main file that points to an existing source file, bailing...`);
  }

  if (!packageJson.name) {
    throw new Error(`package.json at ${packagesRoot}/${packagePath} does not have a valid "name", bailing...`);
  }

  if (!packageJson.version) {
    throw new Error(`package.json at ${packagesRoot}/${packagePath} does not have a valid "version", bailing...`);
  }

  if (packageJson.module) {
    throw new Error(`package.json at ${packagesRoot}/${packagePath} shouldn't have a "module" field, bailing...`);
  }

  if (packageJson.typings) {
    throw new Error(`package.json at ${packagesRoot}/${packagePath} shouldn't have a "typings" field, bailing...`);
  }

  console.log(`building ${packageJson.name} at packages/${packagePath}...`);
  const bundle = await rollup.rollup({
    input: path.join(packagesRoot, packagePath, main),
    external: id => {
      // via tsdx
      // TODO: this should probably be included into deps instead
      if (id === 'babel-plugin-transform-async-to-promises/helpers') {
        // we want to inline these helpers
        return false;
      } // exclude any dependency that's not a realtive import


      return !id.startsWith('.') && !path.isAbsolute(id);
    },
    treeshake: {
      // via tsdx: Don't use getters and setters on plain objects.
      propertyReadSideEffects: false
    },
    plugins: [(0, _pluginNodeResolve.default)({
      extensions,
      browser: true,
      mainFields: ['module', 'main', 'browser']
    }), (0, _pluginCommonjs.default)({
      include: /\/node_modules\//
    }), (0, _pluginBabel.default)({
      babelHelpers: 'bundled',
      presets: [// Preset orders matters, please see: https://github.com/babel/babel/issues/8752#issuecomment-486541662
      ['@babel/preset-env', // TODO: why doesn't this read `targets` from package.json?
      {
        targets: {
          // We should be building packages for environments which support esmodules given their wide support now.
          esmodules: true
        }
      }], ['@babel/preset-typescript', {
        isTSX: true,
        allExtensions: true
      }], '@babel/preset-react'],
      plugins: ['@babel/plugin-proposal-class-properties'],
      extensions,
      include: [`${packagesRoot}/**/*`],
      exclude: 'node_modules/**'
    }), (0, _rollupPluginPostcss.default)({
      extract: false
    }), // TODO: add sass, dotenv
    (0, _pluginJson.default)(), (0, _rollupPluginPreserveShebangs.preserveShebangs)()] // TODO: support for css modules, sass, dotenv,
    // and anything else create-react-app supports
    // (alternatively, disable support for those in apps)

  });
  const outputOptions = {
    freeze: false,
    sourcemap: true // TODO: read this off env

  }; // we're going to use bundle.write() to actually generate the
  // output files, but first we're going to do a scan
  // to validate dependencies and collect some metadata for later

  const {
    output
  } = await bundle.generate(outputOptions); // TODO: we should use this loop to generate the files itself
  // to avoid the second scan, but it's ok for now I guess.
  // "local" workspaces/packages that were imported, i.e - packages/*

  const localImports = {}; // this is used to collect local filenames being referenced
  // to prevent errors where facades are imported as dependencies
  // and are collected in missingDependencies

  const localFileNames = new Set(); // imports that aren't defined in package.json or root package.json
  // Now, this will also mark dependencies that were transient/nested,
  // but I think that's the right choice; a dependency might remove it,
  // even in a patch, and it'll break your code and you wouldn't know why.

  const missingDependencies = new Set();

  for (const chunkOrAsset of output) {
    if (chunkOrAsset.type === 'asset') {// TODO: what should happen here?
    } else {
      // it's a 'chunk' of source code, let's analyse it
      for (const imported of [...chunkOrAsset.imports, ...chunkOrAsset.dynamicImports]) {
        // get the dependency (without references any inner modules)
        const importedPath = imported.split('/');
        const importedPackage = // scoped package?
        importedPath[0][0] === '@' ? `${importedPath[0]}/${importedPath[1]}` : // non-scoped
        importedPath[0];

        if (importedPackage !== imported && packageNames.includes(importedPackage) && // it's fine if it's anything but a js file
        extensions.includes(path.extname(imported))) {
          // TODO: revisit this if and when we have support for multiple entrypoints
          // TODO: add a line number and file name here
          console.error(`cannot import a submodule ${imported} from ${importedPackage}`); // TODO: This could probably be an error, but
          // let's revisit it when we have a better story.
        }

        if (packageJsons[importedPackage]) {
          // This means we're importing from a local workspace
          // Let's collect the name and add it in the package.json
          // we publish to the registry
          // TODO: make sure local workspaces are NOT explicitly included in package.json
          if (packageJsons[importedPackage].private !== true) {
            localImports[importedPackage] = packageJsons[importedPackage].version;
          } else {
            throw new Error(`referencing a private package: ${importedPackage}`); // TODO - lineNo, filename
          }
        } else {
          var _packageJson$dependen, _packageJson$peerDepe;

          // remote
          if ( // not mentioned in the local package.json
          !((_packageJson$dependen = packageJson.dependencies) !== null && _packageJson$dependen !== void 0 && _packageJson$dependen[importedPackage]) && !((_packageJson$peerDepe = packageJson.peerDependencies) !== null && _packageJson$peerDepe !== void 0 && _packageJson$peerDepe[importedPackage])) {
            if (rootPackageJsonDependencies[importedPackage]) {
              localImports[importedPackage] = rootPackageJsonDependencies[importedPackage];
            } else {
              // not mentioned in the root package.json either, so
              // let's collect its name and throw an error later
              // TODO: if it's in root's dev dependencies, should throw a
              // different kind of error
              if (!_builtinModules.default.includes(importedPackage)) {
                // save filename to remove from missingDeps later
                // if they exist there
                localFileNames.add(chunkOrAsset.fileName);
                missingDependencies.add(importedPackage);
              }
            }
          }
        }
      }
    }
  }

  if (Object.keys(localImports).length > 0) {
    console.log('Adding dependencies to the generated package.json:');
    console.log(localImports);
  } // remove local filenames from missingDependencies


  const missingDependenciesWithoutLocalFileNames = [...missingDependencies].filter(dep => !localFileNames.has(dep));

  if (missingDependenciesWithoutLocalFileNames.length > 0) {
    throw new Error(`Missing dependencies: ${missingDependenciesWithoutLocalFileNames.join(', ')};`);
  } // now actually write the bundles to disk
  // TODO: write to disk in the above check itself to prevent this 2nd pass


  await bundle.write({ ...outputOptions,
    ...(preserveModules ? {
      preserveModules: true,
      dir: path.join(packagesRoot, packagePath, `${outputDirectory}-cjs`)
    } : {
      file: path.join(packagesRoot, packagePath, `${outputDirectory}-cjs`, (0, _changeCase.paramCase)(packageJson.name) + '.cjs.js')
    }),
    format: 'cjs',
    exports: 'auto'
  });

  if (!compilingBin) {
    await bundle.write({ ...outputOptions,
      ...(preserveModules ? {
        preserveModules: true,
        dir: path.join(packagesRoot, packagePath, `${outputDirectory}-es`)
      } : {
        file: path.join(packagesRoot, packagePath, `${outputDirectory}-es`, (0, _changeCase.paramCase)(packageJson.name) + '.es.js')
      }),
      format: 'es',
      exports: 'auto'
    });
  }

  let outputFilesPackageJson;

  if (compilingBin && packageJson.bin) {
    const binName = Object.keys(packageJson.bin)[0];
    const binPath = main.replace(/\.tsx?$/, '.js').replace(path.dirname(main) + '/', '');
    outputFilesPackageJson = {
      bin: {
        [binName]: binPath
      }
    };
  } else {
    outputFilesPackageJson = {
      // TODO: what of 'bin' fields?
      main: preserveModules ? path.join(`${outputDirectory}-cjs`, main.replace(/\.tsx?$/, '.js').replace(path.dirname(main) + '/', '')) : `${outputDirectory}-cjs/${(0, _changeCase.paramCase)(packageJson.name) + '.cjs.js'}`,
      module: preserveModules ? path.join(`${outputDirectory}-es`, main.replace(/\.tsx?$/, '.js').replace(path.dirname(main) + '/', '')) : `${outputDirectory}-es/${(0, _changeCase.paramCase)(packageJson.name) + '.es.js'}`,
      typings: path.join(`${outputDirectory}-types`, main.replace(/\.tsx?$/, '.d.ts'))
    };
  } // store the public facing package.json that we'll write to disk later


  publicPackageJsons[packageJson.name] = { ...packageJson,
    ...outputFilesPackageJson,
    dependencies: { ...packageJson.dependencies,
      ...localImports
    },
    files: distinct([...(packageJson.files || []), '/dist-cjs', '/dist-es', '/dist-types', 'README.md'])
  };
  console.log(`built ${packageJson.name} at ${packagePath}`);
  return [true, compilingBin];
}

function makeTypings(packagePath) {
  const console = getConsole(packagePath);
  const packageJson = packageJsonsByPackagePath[packagePath];
  console.log('generating .d.ts files for', packageJson.name); // make a shallow copy of the configuration

  const tsconfig = { ...typescriptConfig,
    compilerOptions: { ...typescriptConfig.compilerOptions
    }
  }; // then add our custom stuff
  // Only include src files from the package to prevent already built
  // files from interferring with the compile

  tsconfig.include = [`${packagesRoot}/${packagePath}/src`];
  tsconfig.compilerOptions = { ...tsconfig.compilerOptions,
    declarationDir: `${packagesRoot}/${packagePath}/${outputDirectory}-types`,
    rootDir: `${packagesRoot}/${packagePath}`
  }; // Extract config information

  const configParseResult = ts.parseJsonConfigFileContent(tsconfig, ts.sys, path.dirname(typescriptConfigFilename));

  if (configParseResult.errors.length > 0) {
    reportTSDiagnostics(packagePath, configParseResult.errors);
    throw new Error('Could not parse Typescript configuration');
  }

  const host = ts.createCompilerHost(configParseResult.options);

  host.writeFile = (fileName, contents) => {
    fse.mkdirpSync(path.dirname(fileName));
    fse.writeFileSync(fileName, contents);
  }; // Compile


  const program = ts.createProgram(configParseResult.fileNames, configParseResult.options, host);
  const emitResult = program.emit(); // Report errors

  const diagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);

  if (diagnostics.length > 0) {
    reportTSDiagnostics(packagePath, diagnostics);
    throw new Error('Could not generate .d.ts files');
  }
}

async function buildPackage(packagePath, preserveModules = false) {
  const modularRoot = (0, _getModularRoot.default)();

  if (process.cwd() !== modularRoot) {
    throw new Error('This command can only be run from the root of a modular project');
  }

  const console = getConsole(packagePath); // ensure the root build folder is ready

  await fse.mkdirp(outputDirectory); // delete any existing local build folders

  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-cjs`));
  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-es`));
  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-types`)); // generate the js files

  const [didBundle, compilingBin] = await makeBundle(packagePath, preserveModules);

  if (!didBundle) {
    return;
  } // then the .d.ts files


  if (!compilingBin) {
    makeTypings(packagePath);
  }

  const originalPkgJsonContent = await fse.readJson(path.join(packagesRoot, packagePath, 'package.json'));
  const packageName = originalPkgJsonContent.name; // switch in the special package.json

  try {
    await fse.writeJson(path.join(packagesRoot, packagePath, 'package.json'), publicPackageJsons[packageName], {
      spaces: 2
    });
    await (0, _execa.default)('yarnpkg', // TODO: verify this works on windows
    ['pack', '--silent', '--filename', path.join(modularRoot, outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz')], {
      cwd: packagesRoot + '/' + packagePath,
      stdin: process.stdin,
      stderr: process.stderr,
      stdout: process.stdout
    });
  } finally {
    // now revert package.json
    await fse.writeJson(path.join(packagesRoot, packagePath, 'package.json'), originalPkgJsonContent, {
      spaces: 2
    });
  } // cool. now unpack the tgz's contents in the root dist


  await fse.mkdirp(path.join(outputDirectory, packagePath));
  await (0, _tar.extract)({
    file: path.join(outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz'),
    strip: 1,
    C: path.join(outputDirectory, packagePath)
  }); // (if you're curious why we unpack it here, it's because
  // we observed problems with publishing tgz files directly to npm.)
  // delete the local dist folders

  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-cjs`));
  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-es`));
  await (0, _util.promisify)(_rimraf.default)(path.join(packagesRoot, packagePath, `${outputDirectory}-types`)); // then delete the tgz

  await fse.remove(path.join(modularRoot, outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz')); /// and... that's it

  console.log('finished');
}
/* TODO:

- build command
  - rm -rf dist && yarn modular build create-modular-react-app,modular-scripts --preserve-modules && yarn workspace modular-views.macro build
- cleanup local dist folders on errors
- read preset-env targets from package.json
  - also, if something _does_ need regenerator, how do we add it as a dep?
- package.json should be able to specify build arguments. Specifically:
  - preserveModules: boolean
  - preserveEntrySignatures:  "strict" | "allow-extension" | "exports-only" | false
- should we disallow using __dirname/__filename in libraries?
- how do we deal with bin fields? maybe inside a standalone bin file,
  we can read package.json's main field?? That could be clever.
- rewrite modular-views.macro with typescript
- how does this work with changesets?
- some kind of build info would be helpful? eg: https://unpkg.com/browse/react@17.0.1/build-info.json
- can we run tests on our built versions? to verify we haven't broken anything.
*/