#!/usr/bin/env node
"use strict";

var _commander = _interopRequireDefault(require("commander"));

var _preflightCheck = _interopRequireDefault(require("./utils/preflightCheck"));

var _build = _interopRequireDefault(require("./build"));

var _addPackage = _interopRequireDefault(require("./addPackage"));

var _start = _interopRequireDefault(require("./start"));

var _test = _interopRequireDefault(require("./test"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
// See https://github.com/facebook/create-react-app/blob/f36d61a/packages/react-scripts/bin/react-scripts.js#L11-L16
process.on('unhandledRejection', err => {
  throw err;
});
const program = new _commander.default.Command('modular');
program.version( // eslint-disable-next-line @typescript-eslint/no-var-requires
require('../package.json').version);
program.command('add <package-name>').description(`Add a new folder into the modular workspace.`).option('--unstable-type <type>', "Type of the folder ('app', 'view', 'package')").option('--unstable-name <name>', 'Package name for the package.json').option('--prefer-offline', 'Equivalent of --prefer-offline for yarn installations', true).action((packageName, addOptions) => {
  return (0, _addPackage.default)(packageName, addOptions.unstableType, addOptions.unstableName, addOptions.preferOffline);
});
program.command('build <packages...>').description('Build a list of packages (multiple package names can be supplied separated by space)').option('--preserve-modules').action(async (packagePaths, options) => {
  console.log('building packages at:', packagePaths.join(', '));

  for (let i = 0; i < packagePaths.length; i++) {
    try {
      await (0, _build.default)(packagePaths[i], options['preserveModules']);
    } catch (err) {
      console.error(`building ${packagePaths[i]} failed`);
      throw err;
    }
  }
});

const testOptions = // eslint-disable-next-line @typescript-eslint/no-var-requires
require('jest-cli/build/cli/args.js').options;

program.command('test [regexes...]').option('--debug', 'Setup node.js debugger on the test process - equivalent of setting --inspect-brk on a node.js process', false).option('--coverage', testOptions.coverage.description).option('--forceExit', testOptions.forceExit.description).option('--env <env>', testOptions.env.description, 'jsdom').option('--maxWorkers <workers>', testOptions.maxWorkers.description).option('--onlyChanged', testOptions.onlyChanged.description).option('--json', testOptions.json.description).option('--outputFile <file>', testOptions.outputFile.description).option('--reporters <reporters...>', testOptions.reporters.description).option('--runInBand', testOptions.runInBand.description).option('--silent', testOptions.silent.description).option('--showConfig', testOptions.showConfig.description).option('--testResultsProcessor <reporter>', testOptions.testResultsProcessor.description).option('--updateSnapshot, -u', testOptions.updateSnapshot.description).option('--verbose', testOptions.verbose.description).option('--watch', testOptions.watch.description).option('--watchAll [value]', testOptions.watchAll.description, !process.env.CI).option('--bail [value]', testOptions.bail.description, process.env.CI).option('--clearCache', testOptions.clearCache.description).option('--logHeapUsage', testOptions.logHeapUsage.description).option('--no-cache', testOptions.cache.description).allowUnknownOption().description('Run tests over the codebase').action((regexes, options) => {
  const {
    U,
    ...testOptions
  } = options; // proxy simplified options to testOptions;

  testOptions.updateSnapshot = !!(options.updateSnapshot || U);
  return (0, _test.default)(testOptions, regexes);
});
program.command('start <packageName>').description(`Start a dev-server for an app. Only available for modular 'app' types.`).action(packageName => {
  return (0, _start.default)(packageName);
});
program.command('workspace').description('Retrieve the information for the current workspace info').action(async () => {
  const {
    getWorkspaceInfo
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('./utils/getWorkspaceInfo')));
  const workspace = await getWorkspaceInfo();
  console.log(JSON.stringify(workspace, null, 2));
});
void (0, _preflightCheck.default)().then(() => {
  return program.parseAsync(process.argv);
}).catch(err => {
  console.error(err);
  process.exit(1);
});