"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _execa = _interopRequireDefault(require("execa"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _isCi = _interopRequireDefault(require("is-ci"));

var _path = _interopRequireDefault(require("path"));

var _updateNotifier = _interopRequireDefault(require("update-notifier"));

var _getWorkspaceInfo = require("../utils/getWorkspaceInfo");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function isYarnInstalled() {
  try {
    await (0, _execa.default)('yarnpkg', ['-v']);
    return true;
  } catch (err) {
    return false;
  }
}

async function preflightCheck() {
  if (process.env.SKIP_PREFLIGHT_CHECK === 'true' || _isCi.default) {
    if (!_isCi.default) {
      console.warn('Skipping modular preflight checks.');
    }
  } else {
    const {
      name,
      version
    } = fs.readJSONSync(_path.default.join(__dirname, '..', '..', 'package.json'));
    const message = 'modular is out of date - you can update from ' + _chalk.default.dim('{currentVersion}') + _chalk.default.reset(' → ') + _chalk.default.green('{latestVersion}') + ' \nRun yarn add modular-scripts@{latestVersion} to update';
    const notifier = (0, _updateNotifier.default)({
      pkg: {
        name: name,
        version: version
      },
      shouldNotifyInNpmScript: true
    });

    if (notifier.update) {
      notifier.notify({
        message,
        defer: false,
        isGlobal: false
      });
    }

    if ((await isYarnInstalled()) === false) {
      throw new Error('Please install `yarn` before attempting to run `modular-scripts`.');
    } // ensure that workspaces are setup correctly with yarn


    const workspace = await (0, _getWorkspaceInfo.getWorkspaceInfo)();

    for (const [packageName, packageInfo] of Object.entries(workspace)) {
      if (packageInfo.mismatchedWorkspaceDependencies.length) {
        throw new Error(`${packageName} has mismatchedWorkspaceDependencies ${packageInfo.mismatchedWorkspaceDependencies.join(', ')}`);
      }
    }
  }

  if (process.env.SKIP_PREFLIGHT_CHECK !== 'true') {
    const verifyPackageTree = // eslint-disable-next-line @typescript-eslint/no-var-requires
    require('react-scripts/scripts/utils/verifyPackageTree');

    verifyPackageTree();
  }
}

var _default = preflightCheck;
exports.default = _default;